@@ -14,6 +14,7 @@ from __future__ import print_function
 import ast
 import inspect
 import os
+import platform
 import re
 import ssl
 import sys
@@ -259,10 +260,16 @@ def locate_app(script_info, module_name, app_name, raise_if_not_found=True):
 def get_version(ctx, param, value):
     if not value or ctx.resilient_parsing:
         return
-    message = 'Flask %(version)s\nPython %(python_version)s'
+    import werkzeug
+    message = (
+        'Python %(python)s\n'
+        'Flask %(flask)s\n'
+        'Werkzeug %(werkzeug)s'
+    )
     click.echo(message % {
-        'version': __version__,
-        'python_version': sys.version,
+        'python': platform.python_version(),
+        'flask': __version__,
+        'werkzeug': werkzeug.__version__,
     }, color=ctx.color)
     ctx.exit()
 
@@ -332,7 +339,7 @@ class DispatchingApp(object):
 
 
 class ScriptInfo(object):
-    """Help object to deal with Flask applications.  This is usually not
+    """Helper object to deal with Flask applications.  This is usually not
     necessary to interface with as it's used internally in the dispatching
     to click.  In future versions of Flask this object will most likely play
     a bigger role.  Typically it's created automatically by the
@@ -340,7 +347,8 @@ class ScriptInfo(object):
     onwards as click object.
     """
 
-    def __init__(self, app_import_path=None, create_app=None):
+    def __init__(self, app_import_path=None, create_app=None,
+                 set_debug_flag=True):
         #: Optionally the import path for the Flask application.
         self.app_import_path = app_import_path or os.environ.get('FLASK_APP')
         #: Optionally a function that is passed the script info to create
@@ -349,6 +357,7 @@ class ScriptInfo(object):
         #: A dictionary with arbitrary data that can be associated with
         #: this script info.
         self.data = {}
+        self.set_debug_flag = set_debug_flag
         self._loaded_app = None
 
     def load_app(self):
@@ -367,7 +376,7 @@ class ScriptInfo(object):
             app = call_factory(self, self.create_app)
         else:
             if self.app_import_path:
-                path, name = (self.app_import_path.split(':', 1) + [None])[:2]
+                path, name = (re.split(r':(?![\\/])', self.app_import_path, 1) + [None])[:2]
                 import_name = prepare_import(path)
                 app = locate_app(self, import_name, name)
             else:
@@ -386,12 +395,10 @@ class ScriptInfo(object):
                 '"app.py" module was not found in the current directory.'
             )
 
-        debug = get_debug_flag()
-
-        # Update the app's debug flag through the descriptor so that other
-        # values repopulate as well.
-        if debug is not None:
-            app.debug = debug
+        if self.set_debug_flag:
+            # Update the app's debug flag through the descriptor so that
+            # other values repopulate as well.
+            app.debug = get_debug_flag()
 
         self._loaded_app = app
         return app
@@ -459,6 +466,8 @@ class FlaskGroup(AppGroup):
     :param load_dotenv: Load the nearest :file:`.env` and :file:`.flaskenv`
         files to set environment variables. Will also change the working
         directory to the directory containing the first file found.
+    :param set_debug_flag: Set the app's debug flag based on the active
+        environment
 
     .. versionchanged:: 1.0
         If installed, python-dotenv will be used to load environment variables
@@ -466,7 +475,8 @@ class FlaskGroup(AppGroup):
     """
 
     def __init__(self, add_default_commands=True, create_app=None,
-                 add_version_option=True, load_dotenv=True, **extra):
+                 add_version_option=True, load_dotenv=True,
+                 set_debug_flag=True, **extra):
         params = list(extra.pop('params', None) or ())
 
         if add_version_option:
@@ -475,6 +485,7 @@ class FlaskGroup(AppGroup):
         AppGroup.__init__(self, params=params, **extra)
         self.create_app = create_app
         self.load_dotenv = load_dotenv
+        self.set_debug_flag = set_debug_flag
 
         if add_default_commands:
             self.add_command(run_command)
@@ -550,7 +561,8 @@ class FlaskGroup(AppGroup):
         obj = kwargs.get('obj')
 
         if obj is None:
-            obj = ScriptInfo(create_app=self.create_app)
+            obj = ScriptInfo(create_app=self.create_app,
+                             set_debug_flag=self.set_debug_flag)
 
         kwargs['obj'] = obj
         kwargs.setdefault('auto_envvar_prefix', 'FLASK')
@@ -632,8 +644,8 @@ def show_server_banner(env, debug, app_import_path, eager_loading):
 
     if env == 'production':
         click.secho(
-            '   WARNING: Do not use the development server in a production'
-            ' environment.', fg='red')
+            '   WARNING: This is a development server. '
+            'Do not use it in a production deployment.', fg='red')
         click.secho('   Use a production WSGI server instead.', dim=True)
 
     if debug is not None:
@@ -670,7 +682,7 @@ class CertParamType(click.ParamType):
 
             obj = import_string(value, silent=True)
 
-            if sys.version_info < (2, 7):
+            if sys.version_info < (2, 7, 9):
                 if obj:
                     return obj
             else:
@@ -687,7 +699,7 @@ def _validate_key(ctx, param, value):
     cert = ctx.params.get('cert')
     is_adhoc = cert == 'adhoc'
 
-    if sys.version_info < (2, 7):
+    if sys.version_info < (2, 7, 9):
         is_context = cert and not isinstance(cert, (text_type, bytes))
     else:
         is_context = isinstance(cert, ssl.SSLContext)
@@ -719,7 +731,7 @@ def _validate_key(ctx, param, value):
     return value
 
 
-@click.command('run', short_help='Runs a development server.')
+@click.command('run', short_help='Run a development server.')
 @click.option('--host', '-h', default='127.0.0.1',
               help='The interface to bind to.')
 @click.option('--port', '-p', default=5000,
@@ -771,10 +783,10 @@ def run_command(info, host, port, reload, debugger, eager_loading,
                threaded=with_threads, ssl_context=cert)
 
 
-@click.command('shell', short_help='Runs a shell in the app context.')
+@click.command('shell', short_help='Run a shell in the app context.')
 @with_appcontext
 def shell_command():
-    """Runs an interactive Python shell in the context of a given
+    """Run an interactive Python shell in the context of a given
     Flask application.  The application will populate the default
     namespace of this shell according to it's configuration.
 
@@ -825,6 +837,10 @@ def routes_command(sort, all_methods):
     """Show all registered routes with endpoints and methods."""
 
     rules = list(current_app.url_map.iter_rules())
+    if not rules:
+        click.echo('No routes were registered.')
+        return
+
     ignored_methods = set(() if all_methods else ('HEAD', 'OPTIONS'))
 
     if sort in ('endpoint', 'rule'):
